@@ -189,6 +189,7 @@ class BackendIMAP extends BackendDiff {
             if ($k == "from") {
                 if (trim($v)) {
                     $changedfrom = true;
+                    $envelopefrom = "-f$v";
                 } elseif (! trim($v) && IMAP_DEFAULTFROM) {
                     $changedfrom = true;
                     if      (IMAP_DEFAULTFROM == 'username') $v = $this->_username;
@@ -361,7 +362,7 @@ class BackendIMAP extends BackendDiff {
                             }
                         }
                     }
-                    if ($multipartmixed) {
+                    if ($multipartmixed && strpos(strtolower($mess2->headers['content-type']), "alternative") !== false) {
                         //this happens if a multipart/alternative message is forwarded
                         //then it's a multipart/mixed message which consists of:
                         //1. text/plain part which was written on the mobile
@@ -392,6 +393,10 @@ class BackendIMAP extends BackendDiff {
         if (!$multipartmixed) {
             if (!empty($forward_h_ct)) $headers .= "\nContent-Type: $forward_h_ct";
             if (!empty($forward_h_cte)) $headers .= "\nContent-Transfer-Encoding: $forward_h_cte";
+            //if body was quoted-printable, convert it again
+            if (isset($message->headers["content-transfer-encoding"]) && strtolower($message->headers["content-transfer-encoding"]) == "quoted-printable") {
+                $body = quoted_printable_encode($body);
+            }
         }
         //advanced debugging
         //debugLog("IMAP-SendMail: parsed message: ". print_r($message,1));
@@ -506,7 +511,7 @@ class BackendIMAP extends BackendDiff {
                 $vars = get_object_vars($overview);
                 if (array_key_exists( "date", $vars)) {
                     // message is out of range for cutoffdate, ignore it
-                    if(strtotime($overview->date) < $cutoffdate) continue;
+                    if ($this->cleanupDate($overview->date) < $cutoffdate) continue;
                     $date = $overview->date;
                 }
 
@@ -545,14 +550,14 @@ class BackendIMAP extends BackendDiff {
             foreach ($list as $val) {
                 $box = array();
                 // cut off serverstring
-                $box["id"] = imap_utf7_decode(substr($val->name, strlen($this->_server)));
+                $box["id"] = substr($val->name, strlen($this->_server));
 
-                $fhir = array_map('imap_utf7_encode',explode($val->delimiter, $box["id"]));
+                $fhir = explode($val->delimiter, $box["id"]);
                 if (count($fhir) > 1) {
                     $this->getModAndParentNames($fhir, $box["mod"], $box["parent"]);
                 }
                 else {
-                    $box["mod"] = imap_utf7_encode($box["id"]);
+                    $box["mod"] = $box["id"];
                     $box["parent"] = "0";
                 }
                 $folders[]=$box;
@@ -623,14 +628,14 @@ class BackendIMAP extends BackendDiff {
 
         // define the rest as other-folders
         else {
-               if (count($fhir) > 1) {
-                   $this->getModAndParentNames($fhir, $folder->displayname, $folder->parentid);
-                   $folder->displayname = windows1252_to_utf8(imap_utf7_decode($folder->displayname));
-               }
-               else {
-                $folder->displayname = windows1252_to_utf8(imap_utf7_decode($id));
+            if (count($fhir) > 1) {
+                $this->getModAndParentNames($fhir, $folder->displayname, $folder->parentid);
+                $folder->displayname = zp_utf7_to_utf8(zp_utf7_iconv_decode($folder->displayname));
+            }
+            else {
+                $folder->displayname = zp_utf7_to_utf8(zp_utf7_iconv_decode($id));
                 $folder->parentid = "0";
-               }
+            }
             $folder->type = SYNC_FOLDER_TYPE_OTHER;
         }
 
@@ -674,6 +679,7 @@ class BackendIMAP extends BackendDiff {
         $this->imap_reopenFolder($folderid);
 
         // build name for new mailbox
+        $displayname = zp_utf7_iconv_encode(zp_utf8_to_utf7($displayname));
         $newname = $this->_server . $folderid . $this->_serverdelimiter . $displayname;
 
         $csts = false;
@@ -782,6 +788,8 @@ class BackendIMAP extends BackendDiff {
             $output = new SyncMail();
 
             $body = $this->getBody($message);
+            $output->bodysize = strlen($body);
+
             // truncate body, if requested
             if(strlen($body) > $truncsize) {
                 $body = utf8_truncate($body, $truncsize);
@@ -792,9 +800,8 @@ class BackendIMAP extends BackendDiff {
             }
             $body = str_replace("\n","\r\n", str_replace("\r","",$body));
 
-            $output->bodysize = strlen($body);
             $output->body = $body;
-            $output->datereceived = isset($message->headers["date"]) ? strtotime($message->headers["date"]) : null;
+            $output->datereceived = isset($message->headers["date"]) ? $this->cleanupDate($message->headers["date"]) : null;
             $output->displayto = isset($message->headers["to"]) ? $message->headers["to"] : null;
             $output->importance = isset($message->headers["x-priority"]) ? preg_replace("/\D+/", "", $message->headers["x-priority"]) : null;
             $output->messageclass = "IPM.Note";
@@ -806,10 +813,19 @@ class BackendIMAP extends BackendDiff {
             $output->reply_to = isset($message->headers["reply-to"]) ? $message->headers["reply-to"] : null;
 
             // Attachments are only searched in the top-level part
-            $n = 0;
             if(isset($message->parts)) {
-                foreach($message->parts as $part) {
-                    if(isset($part->disposition) && ($part->disposition == "attachment" || $part->disposition == "inline")) {
+                $mparts = $message->parts;
+                for ($i=0; $i<count($mparts); $i++) {
+                    $part = $mparts[$i];
+                    //recursively add parts
+                    if($part->ctype_primary == "multipart" && ($part->ctype_secondary == "mixed" || $part->ctype_secondary == "alternative"  || $part->ctype_secondary == "related")) {
+                        foreach($part->parts as $spart)
+                            $mparts[] = $spart;
+                        continue;
+                    }
+                    //add part as attachment if it's disposition indicates so or if it is not a text part
+                    if ((isset($part->disposition) && ($part->disposition == "attachment" || $part->disposition == "inline")) ||
+                        (isset($part->ctype_primary) && $part->ctype_primary != "text")) {
                         $attachment = new SyncAttachment();
 
                         if (isset($part->body))
@@ -824,12 +840,12 @@ class BackendIMAP extends BackendDiff {
                         else $attname = "unknown attachment";
 
                         $attachment->displayname = $attname;
-                        $attachment->attname = $folderid . ":" . $id . ":" . $n;
+                        $attachment->attname = $folderid . ":" . $id . ":" . $i;
                         $attachment->attmethod = 1;
                         $attachment->attoid = isset($part->headers['content-id']) ? $part->headers['content-id'] : "";
                         array_push($output->attachments, $attachment);
                     }
-                    $n++;
+
                 }
             }
             // unset mimedecoder & mail
@@ -1060,9 +1076,9 @@ class BackendIMAP extends BackendDiff {
 
     function enc_multipart($boundary, $body, $body_ct, $body_cte) {
         $mail_body = "This is a multi-part message in MIME format\n\n";
-//        $mail_body .= "--$boundary\n";
-//        $mail_body .= "Content-Type: $body_ct\n";
-//        $mail_body .= "Content-Transfer-Encoding: $body_cte\n\n";
+        $mail_body .= "--$boundary\n";
+        $mail_body .= "Content-Type: $body_ct\n";
+        $mail_body .= "Content-Transfer-Encoding: $body_cte\n\n";
         $mail_body .= "$body\n\n";
 
         return $mail_body;
@@ -1125,6 +1141,18 @@ class BackendIMAP extends BackendDiff {
         return true;
     }
 
+    //if received date has parenthesis (comments) strtotime will return false
+    //the function removes them from the date string
+    function cleanupDate($receiveddate) {
+        $receiveddate = strtotime(preg_replace("/\(.*\)/", "", $receiveddate));
+        if ($receiveddate == false || $receiveddate == -1) {
+            debugLog("Received date is false. Message might be broken.");
+            return null;
+        }
+
+        return $receiveddate;
+    }
+
 }
 
 ?>
\ No newline at end of file
